 
*--------------------------------------------------------------------
* Program: 3,7 exsslogl z rev 4 gp.sas                  
* Author:  John M. Lachin <jml@bsc.gwu.edu>                  
*          Copyright (c) 2015 by John M. Lachin                      
* Source:  Based on exsslogl.sas that is available as part of the 
*		   set of programs for my book on Biostatistical Methods. 
* Purpose: This program determines the sample size to provide a      
*          desired level of power for a test of the equality of the  
*          survival distributions for two groups under an exponential
*          model assuming uniform entry over the period (0,R) and    
*          continued follow-up up to T>R years in all subjects. This 
*          program uses the log hazard ratio as the basis for the    
*          test, rather than difference in hazards. The power        
*          function is presented in equation (2.4) of Lachin and     
*          Foulkes (1986). The program also provides for losses to   
*          follow-up that are also exponentially distributed, using  
*          (4.1) of Lachin and Foulkes.
* 
*          Allows for non-linear recruitment                           
*                                                                    
* Web URL: http://www.bsc.gwu.edu/                                   
*--------------------------------------------------------------------*;
options ls=130 formdlim = '-';
**;
**    to run this program, the steps are:
**       1.  specify the values of the macro variables;
**       2.  create a data set called indata;
**       3.  invoke the macro %ssnz;
**;
**    macro variables:;
**       title = a text description of the design
**       alpha = two-sided type i error probability;
**       qe = sample fractions for the experimental group,;
**            assumed the same for all strata;
**       qc = sample fractions for the control group,;
**            assumed the same for all strata, q**= 1.0 - &qe;
**       power = total power;
**;
**    data set variables, one record per study or scenario;
**       r = period of recruitment;
**       t = total period of study;
**       lc = hazard rate (lc) for the control group;
**       rr = relative hazard in treaed group, le = rr*lc;
**       ldc = hazard rate for losses to followup in the control group;
**       lde = hazard rate for losses to followup in the experimental group;
**;
**;
 
**** new macros for logl version with non-linear recruitment;
 
%macro  num(ht,l,rr,tt,q,g);
   gm = &l*&g*exp(-&ht*&tt) * (1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
   num = 1/(ev*&q);
%mend;
 
%macro  ev(rr,tt,l,ht,g);
   gm = &l*&g*exp(-&ht*&tt)*(1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
%mend;
 
%macro ssnz;
 
title 'sample size for ratio of exponential hazards with exponential losses';
 
data newone; set indata;
      le = rr*lc;
      lh = le*&qe + lc*&qc;
      the = lh + lde;
      thc = lh + ldc;
      te = lde + le;
      tc = ldc + lc;
   **** event probabilities under the alternative;
      %ev(r,t,le,te, gamma);
      pde = ev;
      %ev(r,t,lc,tc, gamma);
      pdc = ev;
   **** event probabilities under the null;
      %ev(r,t,lh,the, gamma);
      pdhe = ev;
      %ev(r,t,lh,thc, gamma);
      pdhc = ev;
   **** loss probabilities;
      %ev(r,t,lde,te, gamma);
      ple = ev;
      %ev(r,t,ldc,tc, gamma);
      plc = ev;
 
*proc print data = one; run;
 
data newtwo; set newone;
beta=1-&power;
za=probit(1 - (&alpha/2));
zb=probit(1 - beta);
      %num(the,lh,r,t,&qe, gamma);
      tnuma = num;
      %num(thc,lh,r,t,&qc, gamma);
      tnumb = num;
      snull = tnuma + tnumb;
      %num(te,le,r,t,&qe, gamma);
      tnuma = num;
      %num(tc,lc,r,t,&qc, gamma);
      tnumb = num;
      salt = tnuma + tnumb;
      nevt= (za + zb)**2 / ( &qe*&qc* (log(le/lc))**2 );
      n = ( (za * sqrt(snull) + zb * sqrt(salt)) / log(rr) )**2;
 
   **** expected # events under the null;
      dhc = pdhc * n * &qc;
      dhe = pdhe * n * &qe;
   **** expected # events and losses under the alternative;
      dc = pdc * n * &qc;
      de = pde * n * &qe;
      nlc = plc * n * &qc;
      nle = ple * n * &qe;
dtot = de + dc;
 
data; set newtwo;
if _n_ = 1 then do;
alph = &alpha;
powr=1-beta;
qfe = &qe; qfc = &qc;
 
file print;
put "&title";
put @5 'type 1 error (2-sided):'       @38 alph 10.5;
put @5 '     z-value (2-sided):'       @38 za     10.5;
put @5 'power:                 '       @38 powr 10.5;
put @5 '     z-value:          '       @38 zb     10.5;
put @5 'sample fractions in e and c'   @38 (qfe qfc) (10.5 10.5);
put @5 'recruitment parameter gamma'   @38 gamma 10.5;
put /;
put
'     periods       sample relative'
       '   --------------- e v e n t s ---------------- '
       '  ---------------- l o s s e s ----------------';
put
' recruit  follow    size   hazard '
       '     hazards      probabilities    exp. number '
       '     hazards      probabilities    exp. number';
put  @34
       ' control treated control treated control treated '
       ' control treated control treated control treated';
end;
 
file print;
put (r t) (8.1 8.1) n 9.2 rr 8.4
    (lc le pdc pde dc de ldc lde plc ple nlc nle)
    (8.4 8.4 8.4 8.4 8.2 8.2 8.4 8.4 8.4 8.4 8.2 8.2);
put 'number events total' @20 nevt 9.2;
 
%mend;
 
data indata;
****;
do R = 3;
gamma = -0.27;
 
do T = 7; 
lc = 0.0875; *, 0.0766;
rr = 0.75;
le = rr*lc;
ld = 0.04;  ***************** NOTE change;
ldc = ld; lde = ld;
 
output; end; end;
 
proc print data=indata; run;
 
%let title = diabetes treatment study- 6 pairwise comparisons;
%let power=0.90;
%let alpha = 0.05/6;  *** two sided value;
%let qc=0.5; %let qe=0.5;
%ssnz;
title 6 pairwise comparisons with 40% at midpoint -- 90% power;
data; set; 
ntot = 4*n/2;
proc print noobs; var gamma r t lc RR lde ldc n dtot dc ntot;
run;
 
proc print noobs; var r t ntot;
run;
 
*********************************************************************************************;
 
**** for a given pattern of recruitment and losses to follow-up, compute the average exposure time.
 
--- using numerical integration
 
--- using the derived expression;
 
*** CORRECTED, COMPUTES TOTAL PROBABILITY TO ENSURE THAT IT EQUALS 1;
 
data three; set newtwo (keep = lc ld R gamma T N);
title E(f) with non-linear recruitment and losses to follow-up;
 
nu = ld; 
Lm = lc; **** control group hazard;
 
**** PLC calculation from SS program;
*ldc == nu; *lc == lm;   
tc = nu + lm;
** %ev(r,t,ldc,tc, gamma) == %ev(r,t,nu,tc, gamma);
**  ev(rr,tt,l,ht,g);
   gm = nu*gamma*exp(-tc*t)*(1 - exp((tc - gamma)*r));
   ev = (nu/tc) + gm/   ( (1 - (exp(-gamma*r))) *tc*(tc - gamma) );
 plc = ev;
 
***** Lachin-Foulkes equations;
**** probability of event; 
ED = (lm/(lm+nu)) + ( lm*gamma*exp(-T*(lm+nu)) * (1 - exp(R*(lm+nu-gamma))))
   / ((lm+nu) * (lm+nu-gamma) * (1 - exp(-R*gamma))  );     
**** probability of lost-to-follow-up; 
EL = (nu/(lm+nu)) + ( nu*gamma*exp(-T*(lm+nu)) * (1 - exp(R*(lm+nu-gamma))))
   / ((lm+nu) * (lm+nu-gamma) * (1 - exp(-R*gamma))  ); 
 
**** another direct calculation as EFd = T - ER where ER is the mean recruitment time;
**** This does not factor losses or events; 
ER = (1 - (exp(-R*gamma))*(1+R*gamma)) / (gamma*(1 - (exp(-R*gamma))));
EFd = T - Er;
 
/*term1 = gamma*nu/((nu**2)*(1-exp(-r*gamma)));
term2 = (1 - exp(-r*gamma))/gamma;
term3 = (r*nu**2 - r*nu*gamma - nu)*exp(r*(nu-gamma))/(nu**2 -2*nu*gamma + gamma**2);
term4 = (nu)/(nu**2 -2*nu*gamma + gamma**2);
term5 = (t*nu + 1)*(exp(-T*nu))*(1 - exp(-r*(gamma-nu)))/(gamma - nu);
Efdirect = term1*(term2 + term3 + term4 - term5);
*/;
 
**** NOTE Ef is the expected maximal exposure time, allowing for losses,  
     but NOT factoring curtailment because of events;
**** Also, Efu is the expected follow-up time allowing for both losses and events; 
 
th = nu+lm;  **** independent competing risks;
dz = 0.01;
du = 0.01;
totalprob = 0;
Ef = 0; Efu = 0;
do z = 0 to R by dz;
pz = gamma*exp(-gamma*z)/(1 - exp(-gamma*R));
do u = 0 to (T-z-du) by du;
pu = nu*exp(-nu*u);  * prob loss to follow-up;
Ef = ef + u*pz*pu*dz*du;
pe = th*exp(-th*u); * probability of either event or loss;
Efu = Efu + u*pz*pe*dz*du;
totalprob = totalprob + pz*pe*dz*du;
end;
Se = exp(-th*(T-z)); ** survives both event and loss to follow-up;
ef = ef + pz*(T-z)*exp(-nu*(T-z))*dz;
efu = efu + pz*(T-z)*(Se)*dz;
totalprob = totalprob + pz*dz*(Se);
end; /* end; end; end;*/
 
proc print; var R T gamma lm nu ER EFd ef efu ED el plc totalprob;* efdirect;
run;
 
***** NOW compute Patient Years;
 
data four; set three;
PYEf = Ef*N;
PYEfu = EFu*N;
 
proc print; var R t N Ef PYeF EFu PYEfu;
run;
 