*--------------------------------------------------------------------
* Program: Expwrlogl z.sas.                                          
* Source:  Based on exsslogl.sas 									 
* Author:  John M. Lachin <jml@bsc.gwu.edu> 		                 
*          Copyright (c) 2015 by John M. Lachin                      
* Source:  Based on exsslogl.sas that is available as part of the 
*		   set of programs for my book on Biostatistical Methods. 
* Purpose: For a given sample size, this program determines the      
*		   level of power for a test of the equality of the          
*          survival distributions for two groups under an exponential
*          model assuming uniform entry over the period (0,R) and    
*          continued follow-up up to T>R years in all subjects. This 
*          program uses the log hazard ratio as the basis for the    
*          test, rather than difference in hazards. The power        
*          function is presented in equation (2.4) of Lachin and     
*          Foulkes (1986). The program also provides for losses to   
*          follow-up that are also exponentially distributed, using  
*          (4.1) of Lachin and Foulkes.                              
*                                                                    
*          GRADE study computations for pairwise and group wise comparisons
*                                                                    
*          Allows for non-linear recruitment                           
*                                                                    
* Web URL: http://www.bsc.gwu.edu/                                   
*--------------------------------------------------------------------*;
options ls=130;
**;
**    to run this program, the steps are:
**       1.  specify the values of the macro variables;
**       2.  create a data set called indata;
**       3.  invoke the macro %ssn;
**;
**    macro variables:;
**       title = a text description of the design
**       alpha = two-sided type i error probability;
**       qe = sample fractions for the experimental group,;
**            assumed the same for all strata;
**       qc = sample fractions for the control group,;
**            assumed the same for all strata, q**= 1.0 - &qe;
**       power = total power;
**;
**    data set variables, one record per study or scenario;
**       r = period of recruitment;
**       t = total period of study;
**       lc = hazard rate (lc) for the control group;
**       rr = relative hazard in treaed group, le = rr*lc;
**       ldc = hazard rate for losses to followup in the control group;
**       lde = hazard rate for losses to followup in the experimental group;
**;
**;
 
%macro  num(ht,l,rr,tt,q,g);
   gm = &l*&g*exp(-&ht*&tt) * (1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
   num = 1/(ev*&q);
%mend;
 
%macro  ev(rr,tt,l,ht,g);
   gm = &l*&g*exp(-&ht*&tt)*(1 - exp((&ht - &g)*&rr));
   ev = (&l/&ht) + gm/   ( (1 - (exp(-&g*&rr))) *&ht*(&ht - &g) );
%mend;
 
%macro Power;
 
title 'POWER for ratio of exponential hazards with exponential losses';
 
data pwone; set indata;
      le = rr*lc;
      lh = le*&qe + lc*&qc;
      the = lh + lde;
      thc = lh + ldc;
      te = lde + le;
      tc = ldc + lc;
   **** event probabilities under the alternative;
      %ev(r,t,le,te, gamma);
      pde = ev;
      %ev(r,t,lc,tc, gamma);
      pdc = ev;
   **** event probabilities under the null;
      %ev(r,t,lh,the, gamma);
      pdhe = ev;
      %ev(r,t,lh,thc, gamma);
      pdhc = ev;
   **** loss probabilities;
      %ev(r,t,lde,te, gamma);
      ple = ev;
      %ev(r,t,ldc,tc, gamma);
      plc = ev;
 
*proc print data = one; run;
 
data pwtwo; set pwone;
za=probit(1 - (&alpha/2));
 
      %num(the,lh,r,t,&qe, gamma);
      tnuma = num;
      %num(thc,lh,r,t,&qc, gamma);
      tnumb = num;
      snull = tnuma + tnumb;
      %num(te,le,r,t,&qe, gamma);
      tnuma = num;
      %num(tc,lc,r,t,&qc, gamma);
      tnumb = num;
      salt = tnuma + tnumb;
***   n = ( (za * sqrt(snull) + zb * sqrt(salt)) / log(rr) )**2;
      zb = (sqrt(n)*abs(log(rr))  - (za * sqrt(snull)) )/sqrt(salt);
	  power = probnorm(zb);
   **** expected # events under the null;
      dhc = pdhc * n * &qc;
      dhe = pdhe * n * &qe;
   **** expected # events and losses under the alternative;
      dc = pdc * n * &qc;
      de = pde * n * &qe;
      nlc = plc * n * &qc;
      nle = ple * n * &qe;
dtot = de + dc;
 
data; set pwtwo;
alpha = &alpha;
if _n_ = 1 then do;
qfe = &qe; qfc = &qc;
 
file print;
put "&title";
put @5 'type 1 error (2-sided):'       @38 alpha 10.5;
put @5 '     z-value (2-sided):'       @38 za     10.5;
put @5 'sample size:           '       @38 N      10.1;
put @5 'sample fractions in e and c'   @38 (qfe qfc) (10.5 10.5);
put @5 'recruitment parameter gamma'   @38 gamma 10.5;
put /;
put
'     periods              relative'
       '   --------------- e v e n t s ---------------- '
       '  ---------------- l o s s e s ----------------';
put
' recruit  follow   power   hazard '
       '     hazards      probabilities    exp. number '
       '     hazards      probabilities    exp. number';
put  @34
       ' control treated control treated control treated '
       ' control treated control treated control treated';
end;
 
file print;
put (r t) (8.1 8.1) power 9.5 rr 8.4
    (lc le pdc pde dc de ldc lde plc ple nlc nle)
    (8.4 8.4 8.4 8.4 8.2 8.2 8.4 8.4 8.4 8.4 8.2 8.2);
 
%mend;
 
 run;
 
%let title = diabetes treatment study;
 
************** **** 6 pairwise comparisons of 4 groups **********************************;
 
data one;
 
************ CHECK RUN: power of pairwise comparisons with N = 5000 from pairwise tests ************";
 
N = 4964/2;  **** 2 groups of 4964/4 per group, 4964/2 being derived from pairwise computation;
%let alpha = 0.05/6;  *** two sided value;
%let qc=0.5; %let qe=0.5;
 
r = 3; t = 7; 
gamma = -0.27; 
do lc = 0.0875;
do rr = 0.75;
le = rr*lc;
do ld = 0.04;
ldc = ld; lde = ld;
output;
end;end;end;
*proc print data=indata; run;
 
data indata; set one;
%Power;
title1 "power of pairwise comparisons with N = 5000 from pairwise tests";
title2 6 pairwise comparison2;
proc print noobs; var alpha gamma r t lc RR lde ldc n zb power dtot;
run;
 
************ power of pairwise comparisons with N = 3300 for the omnibus test ************";
 
data indata; set one;
N = 3300/2;  **** 2 groups of 3300/4 per group;
run;
 
data indata; set one;
%Power;
title1 "power of pairwise comparisons with N = 3300 for the omnibus test";
title2 6 pairwise comparison2;
proc print noobs; var alpha gamma r t lc RR lde ldc n zb power dtot;
run;
 
 
************ power of pairwise comparisons from pooled estimates over strata ************";
 
data indata; set one;
N = 2500; *** 1250 per group;
*** pooled parameters;
lc = (3/5)*0.0875 + (2/5)*0.07;
lrr = (3/5)*log(0.75) + (2/5)*log(0.85);
RR = exp(lrr);
 
proc print;
 
%Power;
title1 "power of pairwise comparisons with N = 5000 for the omnibus test using pooled parameters over 2 strata";
title2 6 pairwise comparison2;
proc print noobs; var alpha gamma r t lc lrr RR lde ldc n zb power dtot;
run;
 
************ 4 comparisons of one group versus rest;
 
%let alpha = 0.05/4;  *** two sided value;
%let qc=0.75; %let qe=0.25;
 
data indata; set one;
N = 3300;
%Power;
 
title2 4 comparisons of each drug versus others combined;
proc print noobs; var alpha gamma r t lc RR lde ldc n zb power dtot;
run;
 
data indata; set one;
N = 3300/2;  **** 2 groups of 3300/4 per group;
run;
 
************************** SPECIAL FOR RESPONSE TO THE DSMB;
 
data indata; set one;
%let alpha = 0.05/6;  *** two sided value;
%let qc=0.5; %let qe=0.5;
 
N = 2000; *** 4000 for the 4 groups;
%Power;
title1 "power of pairwise comparisons with N = 4000 for a single stratum with 80% of the 5000";
title2 6 pairwise comparison2;
proc print noobs; var alpha gamma r t lc RR lde ldc n zb power dtot;
run;
 
 
 